/*
 * Created on 30.11.2004
 *
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package edu.teco.particles.pComponent;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.table.TableColumn;

import edu.teco.particles.pComponent.console.ConsoleGUI;
import edu.teco.particles.pComponent.console.ConsoleTableModel;
import edu.teco.particles.pComponent.io.ParticlePacketListener;
import edu.teco.particles.pComponent.io.ParticlePacketsEvent;

import java.awt.event.*;

/**
 * The Facade of the Console Component.
 * 
 * The Console is a JTable showing some of the Properties of a ParticlePacket.
 * Which properties are shown can be specified with the set<Property>
 * functions.
 * 
 * @author Philipp scholl <scholl@teco.edu>
 */
public class Console extends JPanel implements ParticlePacketListener
{

  protected ConsoleTableModel model = null;

  protected ConsoleGUI        gui   = null;

  /**
   * The Columns of the Model.
   * 
   * The mapping from int<->ColumnObject is hardcoded in the Constructor. We
   * need to know if a Column is en- or disabled. @{
   */
  private TableColumn         _seqNrColumn;

  private boolean             _showSeqNr;

  private TableColumn         _srcIdColumn;

  private boolean             _showSrcIdColumn;

  private TableColumn         _locColumn;

  private boolean             _showLocColumn;

  private TableColumn         _aclColumn;

  private boolean             _showAclColumn;

  private TableColumn         _timeColumn;

  private boolean             _showTimeColumn;

  /* @} */

  /**
   * Creates a new Console Bean.
   * 
   * sets which Column to show by default.
   */
  public Console()
  {
    model = new ConsoleTableModel();
    createGUI();

    _seqNrColumn = gui.getColumnModel().getColumn(0);
    _srcIdColumn = gui.getColumnModel().getColumn(1);
    _locColumn = gui.getColumnModel().getColumn(2);
    _aclColumn = gui.getColumnModel().getColumn(3);
    _timeColumn = gui.getColumnModel().getColumn(4);

    setShowSeqNr(true);
    setShowSourceId(true);
    setShowLocation(false);
    setShowAclTuples(true);
    setShowTimeStamp(false);

    _seqNrColumn.setPreferredWidth(7);
    _srcIdColumn.setPreferredWidth(150);
    _locColumn.setPreferredWidth(300);
    _aclColumn.setPreferredWidth(200);
    _timeColumn.setPreferredWidth(150);
  }

  /**
   * Creates the ConsoleGui Instance.
   * 
   * connects it to this JPanel.
   */
  public void createGUI()
  {
    this.setLayout(new BorderLayout());

    gui = new ConsoleGUI();
    gui.setModel(model);
    this.add(gui, java.awt.BorderLayout.CENTER);

    this.setName("console");
  }

  public void particlePackets(  ParticlePacketsEvent e )
  {
    assert(  _socket != null );
    synchronized (_socket.getBuffer() )
    {
      for (int i=0; i<_socket.getBuffer().size(); i++)
      {
        model.addPacket(_socket.getBuffer().get(i), 
                        _socket.getTimeStamps().get(i));
      }
    }
  }

  /**
   * @name ShowSeqNrProperty
   * @short This is the Property to Show or not Show the SeqNr Column. @{
   */
  public boolean getShowSeqNr()
  {
    return _showSeqNr;
  }

  public void setShowSeqNr(boolean state)
  {
    _showSeqNr = state;
    if (_showSeqNr)
    {
      // we make sure that the column is not duplicated !
      gui.getColumnModel().removeColumn(_seqNrColumn);
      gui.getColumnModel().addColumn(_seqNrColumn);
    }
    else
    {
      gui.getColumnModel().removeColumn(_seqNrColumn);
    }
  }

  /* @} */

  /**
   * @name ShowLocationProperty
   * @short The Location is the 44 Byte long field in the ParticlePacket. @{
   */
  public boolean getShowLocation()
  {
    return _showLocColumn;
  }

  public void setShowLocation(boolean state)
  {
    _showLocColumn = state;
    if (_showLocColumn)
    {
      gui.getColumnModel().removeColumn(_locColumn);
      gui.getColumnModel().addColumn(_locColumn);
    }
    else
    {
      gui.getColumnModel().removeColumn(_locColumn);
    }
  }

  /* @} */

  /**
   * @name ShowSourceIDProperty
   * @short The Source ID is a 8 Byte field in the ParticlePacket. @{
   */
  public boolean getShowSourceId()
  {
    return _showSrcIdColumn;
  }

  public void setShowSourceId(boolean state)
  {
    _showSrcIdColumn = state;
    if (_showSrcIdColumn)
    {
      gui.getColumnModel().removeColumn(_srcIdColumn);
      gui.getColumnModel().addColumn(_srcIdColumn);
    }
    else
    {
      gui.getColumnModel().removeColumn(_srcIdColumn);
    }
  }

  /* @} */

  /**
   * @name ShowAclProperty
   * @short The Acl Tuples are the data contained in a ParticlePacket. See
   * ParticleTuple for further Information. @{
   */
  public boolean getShowAclTuples()
  {
    return _showAclColumn;
  }

  public void setShowAclTuples(boolean state)
  {
    _showAclColumn = state;
    if (_showAclColumn)
    {
      gui.getColumnModel().removeColumn(_aclColumn);
      gui.getColumnModel().addColumn(_aclColumn);
    }
    else
    {
      gui.getColumnModel().removeColumn(_aclColumn);
    }
  }

  /* @} */

  /**
   * @name ShowTimeProperty
   * @short All ParticlePackets get timestamped by the SocketThread. This set
   * wether or not this timestamp should be displayed. @{
   */
  public boolean getShowTimeStamp()
  {
    return _showTimeColumn;
  }

  public void setShowTimeStamp(boolean state)
  {
    _showTimeColumn = state;
    if (_showTimeColumn)
    {
      gui.getColumnModel().removeColumn(_timeColumn);
      gui.getColumnModel().addColumn(_timeColumn);
    }
    else
    {
      gui.getColumnModel().removeColumn(_timeColumn);
    }
  }

  /* @} */

  /** @name SocketThreadProperty
      * @short Property for the SocketThread from which packets are received.
      * @{
   */
  private SocketThread _socket;
  public SocketThread getSocket()
  {
    return _socket;
  }
  public void setSocket (  SocketThread socket )
  {
    assert (  socket != null );
    _socket = socket;
    socket.addParticlePacketListener(  this );
  }
  /* @} */

  /** Clears the model of all data.
   */
  public void clear()
  {
    model.clear();
  }
}
