/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package edu.teco.particles.pComponent;

import java.text.ParseException;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import edu.teco.particles.libParticle.ParticlePacket;
import edu.teco.particles.libParticle.ParticleTuple;
import edu.teco.particles.pComponent.io.ParticlePacketListener;
import edu.teco.particles.pComponent.io.ParticlePacketsEvent;
import edu.teco.particles.pComponent.otDebug.CODFile;

/**
 * Facade of the Over-The-Air-Debug component.
 * 
 * functionality: - read debug tuple from the network. According to this
 * information and the one in the .cod file, prints out some info.
 * 
 * @author Philipp Scholl <scholl@?eco.edu>
 */
public class OtDebug extends JPanel implements ParticlePacketListener
{
  /**
   * @name CODFile
   * @short COD File property. @{
   */
  protected CODFile _codFile;

  public CODFile getCodFile()
  {
    return _codFile;
  }

  public void setCodFile(CODFile codfile)
  {
  }

  /** @} */

  /**
   * @name SocketThreadProperty
   * @short Property for the SocketThread from which packets are received. @{
   */
  private SocketThread _socket;

  public SocketThread getSocket()
  {
    return _socket;
  }

  public void setSocket(SocketThread socket)
  {
    assert (socket != null);
    _socket = socket;
    socket.addParticlePacketListener(this);
  }

  /* @} */

  /**
   * reacts on particlePackets.
   * 
   * Scan for "DBG" tuples in the network, if one is found call parseDBGTuple
   * with the tuple.
   */
  public void particlePackets(ParticlePacketsEvent e)
  {
    List<ParticlePacket> buffer = getSocket().getBuffer();

    // make sure, that a CODFile is specified, if not we can save this step.
    if (getCodFile() == null) return;

    // look for a DBG tuple.
    synchronized (buffer)
    {
      ParticleTuple tuple = null;

      for (ParticlePacket pck : buffer)
      {
        tuple = pck.firstAcl();
        while (tuple != null)
        {
          if (tuple.getAclType() == "DBG")
          {
            try
            {
              parseDBGTuple(tuple);
            }
            catch (ParseException e1)
            {
              // TODO Auto-generated catch block
              e1.printStackTrace();
            }
          }

          tuple = pck.nextAcl(tuple);
        }
      }
    }

  }

  protected JLabel _label = new JLabel();

  public JLabel getView()
  {
    if (_label == null)
    {
      _label = new JLabel();
      add(_label);
    }

    return _label;
  }

  /**
   * parse the DBG tuple.
   * 
   * calls CODFile with the data in the tuple. Updates the view, to reflect this
   * event.
   * 
   * @param tuple
   *          the tuple of DBG type.
   */
  protected void parseDBGTuple(ParticleTuple tuple) throws ParseException
  {
    if (tuple.length() < 4)
    {
      throw new ParseException("tuple is too short.", 0);
    }

    final Byte event = new Byte((byte) tuple.getAclByte(0));
    final Long pointer = new Long(tuple.getAclByte(1) << 16
        | tuple.getAclByte(2) << 8 | tuple.getAclByte(3));

    Runnable updateUI = new Runnable() {
      public void run()
      {
        getView().setText(pointer.toString());
      }
    };

    SwingUtilities.invokeLater(updateUI);
  }
}
