/* Created on 30.11.2004
 *
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */
package edu.teco.particles.pComponent.console;

import java.text.SimpleDateFormat;
import java.util.*;

import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;

import edu.teco.particles.libParticle.*;
import edu.teco.particles.pComponent.util.RingBuffer;

/**
 * Second Implementation of a ConsoleTableModel.
 * 
 * We store Particles in an ArrayList, which is sorted by first arrival. But we
 * return it backward sorted, so newest is at top of the list!
 * 
 * Furthermore we have a rowIndex -> particlePacket property mapping.
 */
public class ConsoleTableModel implements TableModel
{

  private RingBuffer<TableModelListener> _listeners     = new RingBuffer<TableModelListener>(
                                                            10);

  private int                            MAXSTORAGE     = 1000;

  /**
   * describing the data in the Table. keep those in sync with the getValueAt
   * function
   */
  private String[]                       _names         = { "Packet Nr.",
      "SeqNr", "SrcId", "Location", "Acl Tuples", "TimeStamp" };

  private Class[]                        _types         = { Integer.class,
      String.class, String.class, String.class, String[].class, String.class };

  /**
   * holds the data of the ParticlePackets. maps the name of the data to a
   * Collection of this data.
   */
  private RingBuffer<Object>             _storage       = new RingBuffer<Object>(
                                                            MAXSTORAGE);

  /**
   * holds the corresponding timeStamps
   */
  private RingBuffer<String>             _timeStamps    = new RingBuffer<String>(
                                                            MAXSTORAGE);

  /**
   * holds the packetNumbers
   */
  private RingBuffer<Integer>            _packetNumbers = new RingBuffer<Integer>(
                                                            MAXSTORAGE);

  /**
   * create a new Console Table Model
   */
  public ConsoleTableModel()
  {
  }

  public void addTableModelListener(TableModelListener l)
  {
    _listeners.add(l);
  }

  public void removeTableModelListener(TableModelListener l)
  {
    _listeners.remove(l);
  }

  public Class<?> getColumnClass(int columnIndex)
  {
    return _types[columnIndex];
  }

  public int getColumnCount()
  {
    return _names.length;
  }

  public String getColumnName(int columnIndex)
  {
    return _names[columnIndex];
  }

  public int getRowCount()
  {
    return _storage.size();
  }

  /**
   * return the value for the specified cell.
   * 
   * since the data to appear in reverse order, we calculate the offset from the
   * end of _storage list instead of the start.
   * 
   * Also we have two kinds of Objects in our storage, which will be processed
   * differently. First the ParticlePacket Object, and ParticleTuple Objects.
   */
  public synchronized Object getValueAt(int rowIndex, int columnIndex)
  {
    assert (columnIndex <= _names.length);
    assert (rowIndex <= _storage.size());

    Object o = _storage.get(_storage.size() - rowIndex - 1);
    if (o instanceof ParticlePacket)
    {
      ParticlePacket packet = (ParticlePacket) o;
      switch (columnIndex) {
      case 0:
        return _packetNumbers.get(_storage.size() - rowIndex - 1);
      case 1:
        return new Short(packet.getSeq()).toString();
      case 2:
        return packet.getSrcId().toString();
      case 3:
        return packet.getLocation().toString();
      case 4:
        ParticleTuple tuple;
        String str = "";

        if ((tuple = packet.firstAcl()) != null)
        {
          str = tuple.getAclType() + " ";
          for (int i = 0; i < tuple.length(); i++)
          {
            str += " ";
            str += tuple.getAclByte(i);
          }
        }

        return str;
      case 5:
        return _timeStamps.get(_storage.size() - rowIndex - 1);
      }
    }
    else if (o instanceof ParticleTuple)
    {
      if (columnIndex == 0)
      {
        return _packetNumbers.get(_storage.size() - rowIndex - 1);
      }
      if (columnIndex != 4)
      {
        return "";
      }
      else
      {
        ParticleTuple tuple = (ParticleTuple) o;
        String str = tuple.getAclType() + " ";
        for (int i = 0; i < tuple.length(); i++)
        {
          str += " ";
          str += tuple.getAclByte(i);
        }
        return str;
      }
    }

    assert (false) : "not reached";
    return "";
  }

  public boolean isCellEditable(int rowIndex, int columnIndex)
  {
    return false;
  }

  public void setValueAt(Object aValue, int rowIndex, int columnIndex)
  {
    assert (false) : "never ever ever reached (hopefully) !";
  }

  /**
   * adds a packet to the Table Model.
   * 
   * Since we reverse the order in getValueAt(), we have to take care that the
   * ParticleTuples appear *before* the ParticlePacket. Also the buffer is
   * sorted with the newest packet at the end so we also need to reverse it.
   */
  public synchronized void addPacket(ParticlePacket packet, Date date)
  {
    int numberOfRows = 0;
    int packetNr = 0;
    
    try
    {
      packetNr = _packetNumbers.getLast()+1;
    } catch (NoSuchElementException e1) { /*ignore*/ }
      
    List<ParticleTuple> tuples = new ArrayList<ParticleTuple>();

    ParticleTuple next = packet.firstAcl();
    next = packet.nextAcl(next);
    numberOfRows++;
    while (next != null)
    {
      tuples.add(next);
      next = packet.nextAcl(next);
      numberOfRows++;
      _timeStamps.add("");
      _packetNumbers.add(packetNr);
    }

    Collections.reverse(tuples);

    _storage.addAll(tuples);
    _storage.add(packet);
    _timeStamps.add(_dateFormat.format(date));
    _packetNumbers.add(packetNr);

    packetNr++;

    _firePacketsAdded(numberOfRows);
  }

  /** used to format the Timestamp */
  protected SimpleDateFormat _dateFormat = new SimpleDateFormat("HH:MM:ss:SSS");

  /**
   * notifies listeners that a new packet is to be shown.
   * 
   * @param number
   *          the number of added packets.
   */
  private void _firePacketsAdded(int number)
  {
    TableModelEvent event = new TableModelEvent(this, 0, number,
        TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT);

    for (TableModelListener l : _listeners)
    {
      l.tableChanged(event);
    }
  }

  /**
   * notifies listeners that all packets have been deleted.
   */
  private void _firePacketsCleared()
  {
    TableModelEvent event = new TableModelEvent(this);

    for (TableModelListener l : _listeners)
    {
      l.tableChanged(event);
    }
  }

  /**
   * clears the Model of all packets.
   */
  public void clear()
  {
    _storage.clear();
    _timeStamps.clear();
    _firePacketsCleared();
  }
}
