/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights rerved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.teco.particles.pComponent.io;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import edu.teco.particles.libParticle.ParticlePacket;
import edu.teco.particles.libParticle.ParticleSrcId;
import edu.teco.particles.libParticle.ParticleTuple;

/**
 * matches a device by it's sent packets.
 * 
 * You can add TupleTemplates to further narrow the matching.
 */
public class DeviceTemplate implements ParticleTemplate, Serializable
{

  // the id is stored as a string to allow easy serialization of this class.
  protected ParticleSrcId _id;

  public DeviceTemplate(ParticleSrcId srcid)
  {
    assert (srcid != null);
    _id = srcid;
  }

  public ParticleSrcId getId()
  {
    return _id;
  }

  /**
   * matches against a packet.
   * 
   * @param packet
   *          the packet to match against.
   * @return true if the packet matches the template.
   */
  public boolean match(ParticlePacket packet)
  {
    boolean state = false;

    if (packet.getSrcId().equals(_id))
    {
      // all childs must match!
      state = true;
      for (ParticleTemplate p : getChilds())
        if (!p.match(packet)) state = false;
    }

    if (state)
    {
      _lastPacket = packet;
      notifyActionListeners();
    }

    return state;
  }

  protected LinkedList<ParticleTemplate> _templates = new LinkedList<ParticleTemplate>();

  /**
   * add a TupleTemplate to this Template.
   * 
   * @param tupleTemplate
   *          the template to add.
   */
  public void addChild(ParticleTemplate tupleTemplate)
  {
    synchronized (_templates)
    {
      _templates.add(tupleTemplate);
    }
  }

  public LinkedList<ParticleTemplate> getChilds()
  {
    return _templates;
  }

  /**
   * parse a given packet.
   * 
   * Retuns an Array of values of this form, <> marks optional elements. [
   * ParticleSrcId srcid, <1.ChildValue>, <2.ChildValue> ... ]
   * 
   */
  public Object parse(ParticlePacket packet) throws IllegalArgumentException
  {
    synchronized (_templates)
    {
      Object[] arr = new Object[_templates.size() + 1];
      arr[0] = _id;

      for (int i = 0; i < _templates.size(); i++)
      {
        arr[i + 1] = _templates.get(i).parse(packet);
      }

      return arr;
    }
  }

  protected ParticlePacket _lastPacket;

  public Object parse() throws IllegalArgumentException
  {
    return parse(_lastPacket);
  }

  /**
   * returns the timestamp of the last received packet.
   * 
   * @return
   * 
   * TODO: implement this, hehe.
   */
  public Date getTimeStamp()
  {
    return new Date(System.currentTimeMillis());
  }

  /**
   * just adds adressing information to the packet.
   * 
   * @param obj
   *          the obj to be put into the packet.
   * @throws IllegalArgumentException
   *           if the object couldn't be serialized to a format the packet can
   *           hold.
   * 
   */
  public ParticlePacket packet(Object obj, ParticlePacket pck)
      throws IllegalArgumentException
  {
    if (pck == null)
    {
      pck = new ParticlePacket();
      pck.setSrcId(_id);
    }

    try
    {
      if (pck.findFirstAcl("cad") != null) pck.remove(pck.findFirstAcl("cad"));
    }
    catch (IllegalArgumentException e)
    {
      // ignore "no cad tuple in packet"
    }

    _addAdress(pck);

    assert (getChilds().size() != 0 || obj == null) : "don't know how to serialize the given obj.";

    for (ParticleTemplate child : getChilds())
    {
      child.packet(obj, pck);
    }

    return pck;
  }

  /**
   * adds the data for adressed sending to the packet.
   * 
   * @param pck
   *          the packet to add the data to.
   */
  private void _addAdress(ParticlePacket pck)
  {
    ParticleSrcId temp = _id;
    pck.aclAdd("CAD", temp.toArray(), 0);
  }

  public String toString()
  {
    String str = this.getClass() + " " + _id.toString();

    for (ParticleTemplate p : getChilds())
    {
      str += p.toString();
    }

    return str;
  }

  /**
   * @name ActionListener. @{
   */
  private LinkedList<ActionListener> _listeners = new LinkedList<ActionListener>();

  public void addActionListener(ActionListener action)
  {
    _listeners.add(action);
  }

  public void removeActionListener(ActionListener action)
  {
    _listeners.remove(action);
  }

  public List<ActionListener> getActionListeners()
  {
    return _listeners;
  }

  protected void notifyActionListeners()
  {
    ActionEvent event = new ActionEvent(this, 0, MATCH_CMD);
    for (ActionListener listener : _listeners)
    {
      listener.actionPerformed(event);
    }
  }

  /** @} */

  protected Object _userObject = null;

  public void setUserObject(Object o)
  {
    _userObject = o;
  }

  public Object getUserObject()
  {
    return _userObject;
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object other)
  {
    try
    {
      DeviceTemplate obj = (DeviceTemplate) other;
      return obj._id.equals(_id);
    }
    catch (ClassCastException e)
    {
      return false;
    }
  }
}
